@@ -1,6 +1,7 @@
 use self::drop_console::drop_console;
 use self::hoist_decls::DeclHoisterConfig;
 use self::optimize::optimizer;
+use self::optimize::OptimizerState;
 use crate::analyzer::analyze;
 use crate::analyzer::ProgramData;
 use crate::compress::hoist_decls::decl_hoister;
@@ -8,6 +9,7 @@ use crate::debug::dump;
 use crate::debug::invoke;
 use crate::marks::Marks;
 use crate::option::CompressOptions;
+use crate::util::now;
 use crate::util::Optional;
 #[cfg(feature = "pretty_assertions")]
 use pretty_assertions::assert_eq;
@@ -17,7 +19,6 @@ use std::fmt::Debug;
 use std::fmt::Display;
 use std::fmt::Formatter;
 use std::time::Instant;
-use swc_common::comments::Comments;
 use swc_common::pass::CompilerPass;
 use swc_common::pass::Repeat;
 use swc_common::pass::Repeated;
@@ -44,7 +45,6 @@ pub(crate) fn compressor<'a>(
     cm: Lrc<SourceMap>,
     marks: Marks,
     options: &'a CompressOptions,
-    comments: Option<&'a dyn Comments>,
 ) -> impl 'a + JsPass {
     let console_remover = Optional {
         enabled: options.drop_console,
@@ -54,10 +54,10 @@ pub(crate) fn compressor<'a>(
         cm,
         marks,
         options,
-        comments,
         changed: false,
         pass: 0,
         data: None,
+        optimizer_state: Default::default(),
     };
 
     chain!(
@@ -71,10 +71,10 @@ struct Compressor<'a> {
     cm: Lrc<SourceMap>,
     marks: Marks,
     options: &'a CompressOptions,
-    comments: Option<&'a dyn Comments>,
     changed: bool,
     pass: usize,
     data: Option<ProgramData>,
+    optimizer_state: OptimizerState,
 }
 
 impl CompilerPass for Compressor<'_> {
@@ -130,7 +130,7 @@ impl VisitMut for Compressor<'_> {
 
     fn visit_mut_module(&mut self, n: &mut Module) {
         debug_assert!(self.data.is_none());
-        self.data = Some(analyze(&*n));
+        self.data = Some(analyze(&*n, self.marks));
 
         if self.options.passes != 0 && self.options.passes + 1 <= self.pass {
             let done = dump(&*n);
@@ -157,7 +157,7 @@ impl VisitMut for Compressor<'_> {
                 self.pass
             );
 
-            let start_time = Instant::now();
+            let start_time = now();
 
             let mut visitor = expr_simplifier();
             n.visit_mut_with(&mut visitor);
@@ -169,13 +169,15 @@ impl VisitMut for Compressor<'_> {
                 }
             }
 
-            let end_time = Instant::now();
+            if let Some(start_time) = start_time {
+                let end_time = Instant::now();
 
-            log::info!(
-                "compress: expr_simplifier took {:?} (pass = {})",
-                end_time - start_time,
-                self.pass
-            );
+                log::info!(
+                    "compress: expr_simplifier took {:?} (pass = {})",
+                    end_time - start_time,
+                    self.pass
+                );
+            }
 
             if cfg!(feature = "debug") && !visitor.changed() {
                 let simplified = dump(&n.clone().fold_with(&mut fixer(None)));
@@ -193,27 +195,32 @@ impl VisitMut for Compressor<'_> {
         {
             log::info!("compress: Running optimizer (pass = {})", self.pass);
 
-            let start_time = Instant::now();
+            let start_time = now();
             // TODO: reset_opt_flags
             //
             // This is swc version of `node.optimize(this);`.
+
+            self.optimizer_state = Default::default();
+
             let mut visitor = optimizer(
                 self.cm.clone(),
                 self.marks,
                 self.options,
-                self.comments,
                 self.data.as_ref().unwrap(),
+                &mut self.optimizer_state,
             );
             n.visit_mut_with(&mut visitor);
             self.changed |= visitor.changed();
 
-            let end_time = Instant::now();
+            if let Some(start_time) = start_time {
+                let end_time = Instant::now();
 
-            log::info!(
-                "compress: Optimizer took {:?} (pass = {})",
-                end_time - start_time,
-                self.pass
-            );
+                log::info!(
+                    "compress: Optimizer took {:?} (pass = {})",
+                    end_time - start_time,
+                    self.pass
+                );
+            }
             // let done = dump(&*n);
             // log::debug!("===== Result =====\n{}", done);
         }
@@ -225,12 +232,20 @@ impl VisitMut for Compressor<'_> {
                 "".into()
             };
 
-            let start_time = Instant::now();
+            let start_time = now();
 
             let mut v = dead_branch_remover();
             n.map_with_mut(|n| n.fold_with(&mut v));
 
-            let end_time = Instant::now();
+            if let Some(start_time) = start_time {
+                let end_time = Instant::now();
+
+                log::info!(
+                    "compress: dead_branch_remover took {:?} (pass = {})",
+                    end_time - start_time,
+                    self.pass
+                );
+            }
 
             if cfg!(feature = "debug") {
                 let simplified = dump(&*n);
@@ -244,12 +259,6 @@ impl VisitMut for Compressor<'_> {
                 }
             }
 
-            log::info!(
-                "compress: dead_branch_remover took {:?} (pass = {})",
-                end_time - start_time,
-                self.pass
-            );
-
             self.changed |= v.changed();
         }
 
@@ -291,7 +300,7 @@ impl VisitMut for Compressor<'_> {
 
     fn visit_mut_script(&mut self, n: &mut Script) {
         debug_assert!(self.data.is_none());
-        self.data = Some(analyze(&*n));
+        self.data = Some(analyze(&*n, self.marks));
 
         {
             let mut v = decl_hoister(
